@@ -6,12 +6,12 @@ use std::io::Write;
 
 use syn;
 
-use bindgen::config::{Config, Language};
+use bindgen::config::{Config, Language, LayoutConfig};
 use bindgen::declarationtyperesolver::DeclarationTypeResolver;
 use bindgen::dependencies::Dependencies;
 use bindgen::ir::{
-    AnnotationSet, Cfg, ConditionWrite, Documentation, GenericParams, Item, ItemContainer, Path,
-    Repr, ToCondition, Type, Typedef,
+    AnnotationSet, Cfg, ConditionWrite, Constant, Documentation, GenericParams, Item,
+    ItemContainer, Path, Repr, ReprAlign, ReprStyle, ToCondition, Type, Typedef,
 };
 use bindgen::library::Library;
 use bindgen::mangle;
@@ -32,11 +32,13 @@ pub struct Struct {
     pub is_tagged: bool,
     /// Whether this is an enum variant body.
     pub is_enum_variant_body: bool,
+    pub alignment: Option<ReprAlign>,
     pub is_transparent: bool,
     pub tuple_struct: bool,
     pub cfg: Option<Cfg>,
     pub annotations: AnnotationSet,
     pub documentation: Documentation,
+    pub associated_constants: Vec<Constant>,
 }
 
 impl Struct {
@@ -45,25 +47,39 @@ impl Struct {
         !self.fields.is_empty() && self.fields.iter().all(|x| x.1.can_cmp_eq())
     }
 
-    pub fn load(item: &syn::ItemStruct, mod_cfg: &Option<Cfg>) -> Result<Self, String> {
-        let is_transparent = match Repr::load(&item.attrs)? {
-            Repr::C => false,
-            Repr::TRANSPARENT => true,
+    pub fn add_associated_constant(&mut self, c: Constant) {
+        self.associated_constants.push(c);
+    }
+
+    pub fn load(
+        layout_config: &LayoutConfig,
+        item: &syn::ItemStruct,
+        mod_cfg: Option<&Cfg>,
+    ) -> Result<Self, String> {
+        let repr = Repr::load(&item.attrs)?;
+        let is_transparent = match repr.style {
+            ReprStyle::C => false,
+            ReprStyle::Transparent => true,
             _ => {
                 return Err("Struct is not marked #[repr(C)] or #[repr(transparent)].".to_owned());
             }
         };
 
-        let (fields, tuple_struct) = match &item.fields {
-            &syn::Fields::Unit => (Vec::new(), false),
-            &syn::Fields::Named(ref fields) => {
+        // Ensure we can safely represent the struct given the configuration.
+        if let Some(align) = repr.align {
+            layout_config.ensure_safe_to_represent(&align)?;
+        }
+
+        let (fields, tuple_struct) = match item.fields {
+            syn::Fields::Unit => (Vec::new(), false),
+            syn::Fields::Named(ref fields) => {
                 let out = fields
                     .named
                     .iter()
                     .try_skip_map(|x| x.as_ident_and_type())?;
                 (out, false)
             }
-            &syn::Fields::Unnamed(ref fields) => {
+            syn::Fields::Unnamed(ref fields) => {
                 let mut out = Vec::new();
                 let mut current = 0;
                 for field in fields.unnamed.iter() {
@@ -85,6 +101,7 @@ impl Struct {
             fields,
             is_tagged,
             is_enum_variant_body,
+            repr.align,
             is_transparent,
             tuple_struct,
             Cfg::append(mod_cfg, Cfg::load(&item.attrs)),
@@ -99,6 +116,7 @@ impl Struct {
         fields: Vec<(String, Type, Documentation)>,
         is_tagged: bool,
         is_enum_variant_body: bool,
+        alignment: Option<ReprAlign>,
         is_transparent: bool,
         tuple_struct: bool,
         cfg: Option<Cfg>,
@@ -113,11 +131,13 @@ impl Struct {
             fields,
             is_tagged,
             is_enum_variant_body,
+            alignment,
             is_transparent,
             tuple_struct,
             cfg,
             annotations,
             documentation,
+            associated_constants: vec![],
         }
     }
 
@@ -160,6 +180,7 @@ impl Struct {
                 .collect(),
             self.is_tagged,
             self.is_enum_variant_body,
+            self.alignment,
             self.is_transparent,
             self.tuple_struct,
             self.cfg.clone(),
@@ -167,6 +188,45 @@ impl Struct {
             self.documentation.clone(),
         )
     }
+
+    fn emit_bitflags_binop<F: Write>(
+        &self,
+        operator: char,
+        other: &str,
+        out: &mut SourceWriter<F>,
+    ) {
+        out.new_line();
+        write!(
+            out,
+            "{} operator{}(const {}& {}) const",
+            self.export_name(),
+            operator,
+            self.export_name(),
+            other
+        );
+        out.open_brace();
+        write!(
+            out,
+            "return {{static_cast<decltype(bits)>(this->bits {} {}.bits)}};",
+            operator, other
+        );
+        out.close_brace(false);
+
+        out.new_line();
+        write!(
+            out,
+            "{}& operator{}=(const {}& {})",
+            self.export_name(),
+            operator,
+            self.export_name(),
+            other
+        );
+        out.open_brace();
+        write!(out, "*this = (*this {} {});", operator, other);
+        out.new_line();
+        write!(out, "return *this;");
+        out.close_brace(false);
+    }
 }
 
 impl Item for Struct {
@@ -178,8 +238,8 @@ impl Item for Struct {
         &self.export_name
     }
 
-    fn cfg(&self) -> &Option<Cfg> {
-        &self.cfg
+    fn cfg(&self) -> Option<&Cfg> {
+        self.cfg.as_ref()
     }
 
     fn annotations(&self) -> &AnnotationSet {
@@ -266,6 +326,10 @@ impl Item for Struct {
         for field in &mut self.fields {
             reserved::escape(&mut field.0);
         }
+
+        for c in self.associated_constants.iter_mut() {
+            c.rename_for_config(config);
+        }
     }
 
     fn add_dependencies(&self, library: &Library, out: &mut Dependencies) {
@@ -279,6 +343,10 @@ impl Item for Struct {
         for &(_, ref ty, _) in fields {
             ty.add_dependencies_ignoring_generics(&self.generic_params, library, out);
         }
+
+        for c in &self.associated_constants {
+            c.add_dependencies(library, out);
+        }
     }
 
     fn instantiate_monomorph(
@@ -327,7 +395,12 @@ impl Source for Struct {
                 annotations: self.annotations.clone(),
                 documentation: self.documentation.clone(),
             };
-            return typedef.write(config, out);
+            typedef.write(config, out);
+            for constant in &self.associated_constants {
+                out.new_line();
+                constant.write(config, out, Some(self));
+            }
+            return;
         }
 
         let condition = (&self.cfg).to_condition(config);
@@ -352,6 +425,27 @@ impl Source for Struct {
 
         out.write("struct");
 
+        if let Some(align) = self.alignment {
+            match align {
+                ReprAlign::Packed => {
+                    if let Some(ref anno) = config.layout.packed {
+                        write!(out, " {}", anno);
+                    }
+                }
+                ReprAlign::Align(n) => {
+                    if let Some(ref anno) = config.layout.aligned_n {
+                        write!(out, " {}({})", anno, n);
+                    }
+                }
+            }
+        }
+
+        if self.annotations.must_use {
+            if let Some(ref anno) = config.structure.must_use {
+                write!(out, " {}", anno);
+            }
+        }
+
         if config.language == Language::Cxx || config.style.generate_tag() {
             write!(out, " {}", self.export_name());
         }
@@ -418,6 +512,32 @@ impl Source for Struct {
                 String::from("other")
             };
 
+            if self
+                .annotations
+                .bool("internal-derive-bitflags")
+                .unwrap_or(false)
+            {
+                if !wrote_start_newline {
+                    wrote_start_newline = true;
+                    out.new_line();
+                }
+                out.new_line();
+                write!(out, "explicit operator bool() const");
+                out.open_brace();
+                write!(out, "return !!bits;");
+                out.close_brace(false);
+
+                out.new_line();
+                write!(out, "{} operator~() const", self.export_name());
+                out.open_brace();
+                write!(out, "return {{static_cast<decltype(bits)>(~bits)}};");
+                out.close_brace(false);
+
+                self.emit_bitflags_binop('|', &other, out);
+                self.emit_bitflags_binop('&', &other, out);
+                self.emit_bitflags_binop('^', &other, out);
+            }
+
             let skip_fields = if self.is_tagged { 1 } else { 0 };
 
             let mut emit_op = |op, conjuc| {
@@ -484,6 +604,16 @@ impl Source for Struct {
             out.write_raw_block(body);
         }
 
+        if config.language == Language::Cxx
+            && config.structure.associated_constants_in_body
+            && config.constant.allow_static_const
+        {
+            for constant in &self.associated_constants {
+                out.new_line();
+                constant.write_declaration(config, out, self);
+            }
+        }
+
         if config.language == Language::C && config.style.generate_typedef() {
             out.close_brace(false);
             write!(out, " {};", self.export_name());
@@ -491,6 +621,11 @@ impl Source for Struct {
             out.close_brace(true);
         }
 
+        for constant in &self.associated_constants {
+            out.new_line();
+            constant.write(config, out, Some(self));
+        }
+
         condition.write_after(config, out);
     }
 }
@@ -504,7 +639,7 @@ impl SynFieldHelpers for syn::Field {
         let ident = self
             .ident
             .as_ref()
-            .ok_or(format!("field is missing identifier"))?
+            .ok_or_else(|| "field is missing identifier".to_string())?
             .clone();
         let converted_ty = Type::load(&self.ty)?;
 
