@@ -1,9 +1,14 @@
 #include <cstdarg>
 #include <cstdint>
 #include <cstdlib>
+#include <new>
 
 static const int32_t PREFIX_LEN = 42;
 
+static const int64_t PREFIX_X = (42 << 42);
+
+static const int64_t PREFIX_Y = (PREFIX_X + PREFIX_X);
+
 using PREFIX_NamedLenArray = int32_t[PREFIX_LEN];
 
 using PREFIX_ValuedLenArray = int32_t[42];
